@@ -46,6 +46,7 @@ struct obj {
 	size_t buffer_size;
 	struct ring ring;
 	int (*header_func)(struct obj *self, GstBuffer *buf);
+	uint64_t next_timestamp;
 };
 
 struct obj_class {
@@ -150,6 +151,16 @@ vorbis_header(struct obj *self, GstBuffer *buf)
 	return 1;
 }
 
+static int
+flac_header(struct obj *self, GstBuffer *buf)
+{
+	unsigned type = (buf->data[0] & 0x7F);
+	/* frame sync code */
+	if (type == 127)
+		return 0;
+	return 1;
+}
+
 static inline void
 calculate_timestamp(struct obj *self, GstBuffer *out_buf)
 {
@@ -175,7 +186,7 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 		int hdr = self->header_func(self, buf);
 		if (!hdr) {
 			self->got_header = true;
-			if (avcodec_open(self->av_ctx, self->codec) < 0) {
+			if (gst_av_codec_open(self->av_ctx, self->codec) < 0) {
 				ret = GST_FLOW_ERROR;
 				goto leave;
 			}
@@ -209,19 +220,26 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 		memcpy(pkt.data, buf->data, buf->size);
 		memset(pkt.data + pkt.size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
 
-		if (G_UNLIKELY(self->timestamp == GST_CLOCK_TIME_NONE))
-			self->timestamp = buf->timestamp;
+		if (G_UNLIKELY(self->timestamp == GST_CLOCK_TIME_NONE)) {
+			self->next_timestamp = self->timestamp = buf->timestamp;
+		} else if (self->next_timestamp != buf->timestamp) {
+			GST_WARNING_OBJECT(self, "reseting timestamp");
+			self->next_timestamp = self->timestamp = buf->timestamp;
+		}
+
+		self->next_timestamp += buf->duration;
 
 		do {
 			void *buffer_data;
 			int buffer_size;
 			int read;
+			unsigned total_buffer_size;
 
 			buffer_data = self->buffer_data + self->ring.in;
 			buffer_size = self->buffer_size - self->ring.in;
 			read = avcodec_decode_audio3(self->av_ctx, buffer_data, &buffer_size, &pkt);
 			if (read < 0) {
-				ret = GST_FLOW_ERROR;
+				GST_WARNING_OBJECT(self, "error: %i", read);
 				break;
 			}
 
@@ -234,9 +252,14 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 				self->ring.out = 0;
 			}
 
-			if (self->ring.in - self->ring.out >= BUFFER_SIZE) {
+			if (BUFFER_SIZE > 0)
+				total_buffer_size = BUFFER_SIZE;
+			else
+				total_buffer_size = buffer_size;
+
+			if (self->ring.in - self->ring.out >= total_buffer_size) {
 				GstBuffer *out_buf;
-				out_buf = gst_buffer_new_and_alloc(BUFFER_SIZE);
+				out_buf = gst_buffer_new_and_alloc(total_buffer_size);
 				memcpy(out_buf->data, self->buffer_data + self->ring.out, out_buf->size);
 				calculate_timestamp(self, out_buf);
 				gst_buffer_set_caps(out_buf, self->srcpad->caps);
@@ -285,7 +308,7 @@ change_state(GstElement *element, GstStateChange transition)
 	switch (transition) {
 	case GST_STATE_CHANGE_READY_TO_NULL:
 		if (self->av_ctx) {
-			avcodec_close(self->av_ctx);
+			gst_av_codec_close(self->av_ctx);
 			av_freep(&self->av_ctx);
 		}
 		av_free_packet(&self->pkt);
@@ -345,10 +368,11 @@ sink_setcaps(GstPad *pad, GstCaps *caps)
 		GstBuffer *buf;
 
 		codec_id = CODEC_ID_FLAC;
+		self->header_func = flac_header;
 
 		stream_header = gst_structure_get_value(in_struc, "streamheader");
 		if (!stream_header)
-			return false;
+			return true;
 
 		stream_info = gst_value_array_get_value(stream_header, 0);
 
@@ -411,14 +435,6 @@ generate_sink_template(void)
 
 	gst_caps_append_structure(caps, struc);
 
-	struc = gst_structure_new("audio/mpeg",
-			"mpegversion", G_TYPE_INT, 1,
-			"layer", G_TYPE_INT, 3,
-			"parsed", G_TYPE_BOOLEAN, TRUE,
-			NULL);
-
-	gst_caps_append_structure(caps, struc);
-
 	return caps;
 }
 
